/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor.resources;

import com.sun.jersey.test.framework.WebAppDescriptor;
import com.sun.jersey.test.framework.AppDescriptor;
import com.sun.jersey.test.framework.JerseyTest;
import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.Note;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.List;
import java.util.ResourceBundle;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class ChatTest extends JerseyTest {
    static {
        System.setProperty("JERSEY_HTTP_PORT", "33437");
    }
    private File dir;

    @Override
    protected AppDescriptor configure() {
        try {
            dir = File.createTempFile("quoridor", ".dir");
            dir.delete();
            System.setProperty("quoridor.dir", dir.getPath());
            dir.mkdirs();
            File passwd = new File(dir, "passwd");
            FileOutputStream os = new FileOutputStream(passwd);
            os.write("Jarda=heslo\nJirka=pesko\n".getBytes("UTF-8"));
            os.close();
        } catch (Exception ex) {
            throw new IllegalStateException(ex);
        }
        return new WebAppDescriptor.Builder("cz.xelfi.quoridor.webidor.resources").build();
    }

    @Override
    public void tearDown() throws Exception {
        super.tearDown();
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    @Test public void testCreateAGame() throws Exception {
        String logJarda = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = resource().path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
    GameId s = resource().path("games").queryParam("loginID", logJarda).
                queryParam("white", "Jarda")
                .queryParam("black", "Jirka").post(GameId.class);

        Thread.sleep(100);
        final long now = System.currentTimeMillis();
        if (s.getModified() >= now) {
            fail("The game is supposed to be modified in past");
        }
        Thread.sleep(100);

        ResourceBundle b = ResourceBundle.getBundle("cz/xelfi/quoridor/webidor/TestBundle");
        String comment = b.getString("COMMENT");

        GameId s1 = resource().path("games/" + s.getId()).
            queryParam("loginID", logJarda).
            queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);

        GameId comment1 = resource().path("games/" + s.getId()).
            queryParam("loginID", logJarda).
            queryParam("player", "Jarda").queryParam("comment", "I like this game!").put(GameId.class);
        assertEquals("One comment in the game", 1, comment1.getComments());

        GameId comment2 = resource().path("games/" + s.getId()).
            queryParam("loginID", logJirka).
            queryParam("player", "Jirka").queryParam("comment", comment).put(GameId.class);
        assertEquals("Snd comment in the game", 2, comment2.getComments());

        GameId s2 = resource().path("games/" + s.getId()).
            queryParam("loginID", logJirka).
            queryParam("player", "Jirka").queryParam("move", "S").put(GameId.class);
        assertNotNull("Successful move", s2);
        assertEquals("Still two comments in the game", 2, s2.getComments());

        File game = new File(new File(dir, "games"), s1.getId());
        assertTrue("File for game exists", game.exists());
        String content = readFile(game);

        if (!content.contains("# white: Jarda")) {
            fail(content);
        }
        if (!content.contains("# black: Jirka")) {
            fail(content);
        }
        if (!content.contains("N")) {
            fail(content);
        }
        if (!content.contains("... S")) {
            fail(content);
        }
        if (!content.contains("I like")) {
            fail(content);
        }
        if (!content.contains(comment)) {
            fail(content);
        }

        Games read = new Games(new File(dir, "games"), new Quoridor());
        List<Game> readGames = read.getGames();
        assertEquals("One game read", 1, readGames.size());
        Board board = readGames.get(0).getBoard();
        assertEquals(1, board.getPlayers().get(0).getRow());
        assertEquals(7, board.getPlayers().get(1).getRow());
        assertEquals(Move.NORTH, readGames.get(0).getMoves().get(0).getMove());
        assertEquals(Move.SOUTH, readGames.get(0).getMoves().get(1).getMove());

        Game rg = readGames.get(0);
        assertNull("No comments on second move", rg.getMoves().get(1).getComments());
        List<Note> cmnts = rg.getMoves().get(0).getComments();
        assertNotNull("Some comments on first move", cmnts);
        assertEquals("Two comments: " + cmnts, 2, cmnts.size());

        if (!cmnts.get(0).getComment().contains("I like")) {
            fail();
        }
        if (!cmnts.get(1).getComment().contains(comment)) {
            fail();
        }

        Thread.sleep(1500);

        File tmp = File.createTempFile("test-board", "game");
        read.storeGame(rg, tmp);

        String sss1 = readFile(game).replaceAll("@.*:", "date");
        Thread.sleep(1500);

        assertEquals("Newly written file remains the same", sss1, readFile(tmp).replaceAll("@.*:", "date"));

        String sGame = resource().path("games").path(s.getId()).accept(MediaType.TEXT_XML).get(String.class);
        Game readGame = resource().path("games").path(s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
        assertNotNull("Game really returned", readGame);

        assertEquals(Move.NORTH, readGame.getMoves().get(0).getMove());
        assertEquals(Move.SOUTH, readGame.getMoves().get(1).getMove());

        assertNull("No comments on second move", readGame.getMoves().get(1).getComments());
        cmnts = readGame.getMoves().get(0).getComments();
        assertNotNull("Some comments on first move", cmnts);
        assertEquals("Two comments: " + cmnts, 2, cmnts.size());
        if (!cmnts.get(0).getComment().contains("I like")) {
            fail();
        }
        if (!cmnts.get(1).getComment().contains(comment)) {
            fail();
        }
    }

    private String readFile(File game) throws IOException, FileNotFoundException {
        char[] arr = new char[4096];
        Reader gameContent = new InputStreamReader(new FileInputStream(game), "UTF-8");
        int len = gameContent.read(arr);
        String content = new String(arr, 0, len);
        return content;
    }

}
